/*********************************************
This program was produced by the
CodeWizardAVR V1.24.0 Standard
Automatic Program Generator
 Copyright 1998-2003 HP InfoTech s.r.l.
http://www.hpinfotech.ro
e-mail:office@hpinfotech.ro

Project : PPE Robot(s)
Version : 1
Date    : 04/07/2009
Author  : MNO                             
Company : Lyce Pierre Emile Martin 18000 BOURGES                                
Comments: Mise en oeuvre du protocole I2C sur l'interface TWI
(Cde des LEDs de la carte relies  un PCF8574)
Adaptation de la note d'application AVR315 d'ATMEL

Chip type           : ATmega8535
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 128
*********************************************/

#include <mega8535.h>
#include <delay.h>

#define TWI_TWBR            	0x20	// 0x20 100kHz0k 
#define TWI_BUFFER_SIZE 	4   	// Set this to the largest message size that will be sent including address byte.    
#define TWI_NO_STATE            0xF8  	// No relevant state information available; TWINT = 0

#define TWI_READ_BIT  0       // Bit position for R/W bit in "address byte".
#define TWI_ADR_BITS  1       // Bit position for LSB of the slave address bits in the init byte.

#define TRUE          1
#define FALSE         0

/****************************************************************************
  TWI State codes
****************************************************************************/
// General TWI Master staus codes                      
#define TWI_START                  0x08  // START has been transmitted  
#define TWI_REP_START              0x10  // Repeated START has been transmitted
#define TWI_ARB_LOST               0x38  // Arbitration lost

// TWI Master Transmitter status codes                      
#define TWI_MTX_ADR_ACK            0x18  // SLA+W has been tramsmitted and ACK received
#define TWI_MTX_ADR_NACK           0x20  // SLA+W has been tramsmitted and NACK received 
#define TWI_MTX_DATA_ACK           0x28  // Data byte has been tramsmitted and ACK received
#define TWI_MTX_DATA_NACK          0x30  // Data byte has been tramsmitted and NACK received 

// TWI Master Receiver staus codes  
#define TWI_MRX_ADR_ACK            0x40  // SLA+R has been tramsmitted and ACK received
#define TWI_MRX_ADR_NACK           0x48  // SLA+R has been tramsmitted and NACK received
#define TWI_MRX_DATA_ACK           0x50  // Data byte has been received and ACK tramsmitted
#define TWI_MRX_DATA_NACK          0x58  // Data byte has been received and NACK tramsmitted

// TWI Slave Transmitter staus codes
#define TWI_STX_ADR_ACK            0xA8  // Own SLA+R has been received; ACK has been returned
#define TWI_STX_ADR_ACK_M_ARB_LOST 0xB0  // Arbitration lost in SLA+R/W as Master; own SLA+R has been received; ACK has been returned
#define TWI_STX_DATA_ACK           0xB8  // Data byte in TWDR has been transmitted; ACK has been received
#define TWI_STX_DATA_NACK          0xC0  // Data byte in TWDR has been transmitted; NOT ACK has been received
#define TWI_STX_DATA_ACK_LAST_BYTE 0xC8  // Last data byte in TWDR has been transmitted (TWEA = 0); ACK has been received

// TWI Slave Receiver staus codes
#define TWI_SRX_ADR_ACK            0x60  // Own SLA+W has been received ACK has been returned
#define TWI_SRX_ADR_ACK_M_ARB_LOST 0x68  // Arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned
#define TWI_SRX_GEN_ACK            0x70  // General call address has been received; ACK has been returned
#define TWI_SRX_GEN_ACK_M_ARB_LOST 0x78  // Arbitration lost in SLA+R/W as Master; General call address has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_ACK       0x80  // Previously addressed with own SLA+W; data has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_NACK      0x88  // Previously addressed with own SLA+W; data has been received; NOT ACK has been returned
#define TWI_SRX_GEN_DATA_ACK       0x90  // Previously addressed with general call; data has been received; ACK has been returned
#define TWI_SRX_GEN_DATA_NACK      0x98  // Previously addressed with general call; data has been received; NOT ACK has been returned
#define TWI_SRX_STOP_RESTART       0xA0  // A STOP condition or repeated START condition has been received while still addressed as Slave

// TWI Miscellaneous status codes

#define TWI_BUS_ERROR              0x00  // Bus error due to an illegal START or STOP condition     


/****************************************************************************
  Function definitions
****************************************************************************/
void TWI_Master_Initialise( void );
unsigned char TWI_Transceiver_Busy( void );
void TWI_Start_Transc_With_Data( unsigned char * , unsigned char );


// Declare your global variables here
//****************************************************************************
static unsigned char TWI_buf[ TWI_BUFFER_SIZE ];    // Transceiver buffer
  static unsigned char TWI_msgSize;                   // Number of bytes to be transmitted.
  static unsigned char TWI_state = TWI_NO_STATE;      // State byte. Default set to TWI_NO_STATE
union TWI_STR                      // Status byte holding flags.
{
    unsigned char all;
    unsigned char lastTransOK:1;      
    unsigned char unusedBits:7;
} TWI_statusReg={0}; 


interrupt [TWI] void twi_isr(void)      // Adapte pour CVAVR
{					// #pragma vector=TWI_vect
					// __interrupt void TWI_ISR(void)

  static unsigned char TWI_bufPtr;
  
  switch (TWSR)
  {
    case TWI_START:             // START has been transmitted  
    
    case TWI_REP_START:         // Repeated START has been transmitted
      TWI_bufPtr = 0;   	// Set buffer pointer to the TWI Address location
      				
    case TWI_MTX_ADR_ACK:       // SLA+W has been tramsmitted and ACK received 
    
    case TWI_MTX_DATA_ACK:      // Data byte has been tramsmitted and ACK received
      if (TWI_bufPtr < TWI_msgSize)
      {
        TWDR = TWI_buf[TWI_bufPtr++];                     // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                                // (1<<TWEN)| TWI Interface enabled
                                     			  // (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag to send byte
                         				  // (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| 
               						  // (0<<TWWC); 
      }else                    // Send STOP after last byte
      {
        TWI_statusReg.lastTransOK = TRUE;                 // Set status bits to completed successfully.
         						  // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE 
        TWCR = 0b10010100; 		                  // (1<<TWEN)| TWI Interface enabled
                                    			  // (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                         				  // (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                               				  // (0<<TWWC);
      }
    break;
      
    case TWI_MRX_DATA_ACK:      // Data byte has been received and ACK tramsmitted
      TWI_buf[TWI_bufPtr++] = TWDR; 
      
    case TWI_MRX_ADR_ACK:       // SLA+R has been tramsmitted and ACK received
      if (TWI_bufPtr < (TWI_msgSize-1) )                  // Detect the last byte to NACK it.
      {                                                   // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                                // (1<<TWEN)| TWI Interface enabled
                                     			  // (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  // (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|Send ACK after reception
                                                          // (0<<TWWC);  
      }else                    // Send NACK after next reception
      {						          // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                                // (1<<TWEN)| TWI Interface enabled
                                     			  // (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  // (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| Send NACK after reception
                                			  // (0<<TWWC); 
      }    
    break; 
      
    case TWI_MRX_DATA_NACK:     // Data byte has been received and NACK tramsmitted
      TWI_buf[TWI_bufPtr] = TWDR;
      TWI_statusReg.lastTransOK = TRUE;                 // Set status bits to completed successfully. 
      							// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10010100;                                // (1<<TWEN)| TWI Interface enabled
                                  			// (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                       					// (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                                             		// (0<<TWWC); 
    break;  
          
    case TWI_ARB_LOST:          // Arbitration lost                             
    							// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10100101;                                // (1<<TWEN)| TWI Interface enabled
                                   			// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag
                        				// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)| Initiate a (RE)START condition.
                                              		// (0<<TWWC);
    break;
      
    case TWI_MTX_ADR_NACK:      // SLA+W has been tramsmitted and NACK received
    case TWI_MRX_ADR_NACK:      // SLA+R has been tramsmitted and NACK received    
    case TWI_MTX_DATA_NACK:     // Data byte has been tramsmitted and NACK received
//    case TWI_NO_STATE              // No relevant state information available; TWINT = 0
    case TWI_BUS_ERROR:         // Bus error due to an illegal START or STOP condition  
    
    default:     
      TWI_state = TWSR;                                 // Store TWSR and automatically sets clears noErrors bit.
                                                        // Reset TWI Interface
                                                        // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b00000100;                                // (1<<TWEN)| Enable TWI-interface and release TWI pins
                                   			// (0<<TWIE)|(0<<TWINT)| Disable Interupt
                       					// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests
                                              		// (0<<TWWC);
  }
}
void main(void)
{
// Declare your local variables here
unsigned char messageLEDOn[]={0x40,0x55}; 
unsigned char messageLEDOff[]={0x40,0xff}; 

TWI_Master_Initialise();
// Global enable interrupts
#asm("sei")

while (1)
      {
        TWI_Start_Transc_With_Data(messageLEDOn,2); 
        delay_ms(200);
	TWI_Start_Transc_With_Data(messageLEDOff,2);  
	delay_ms(200);
      };
}

// Fonctions

void TWI_Master_Initialise(void)
{                                               // Adapte pour CVAVR
  TWBR = TWI_TWBR;                              // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
  TWDR = 0xFF;                                  // Default content = SDA released.
  						//  TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR = 0b00000100;                            // (1<<TWEN)| Enable TWI-interface and release TWI pins.
                               			// (0<<TWIE)|(0<<TWINT)|Disable Interupt.
                               			// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests.
                                         	// 0<<TWWC); 
} 

unsigned char TWI_Transceiver_Busy( void )
{                                               // Adapte pour CVAVR
  return ( TWCR & 0b00000001);                  // ( TWCR & (1<<TWIE) ) IF TWI Interrupt is enabled then the Transceiver is busy
}

void TWI_Start_Transc_With_Data( unsigned char *msg, unsigned char msgSize )
{
  unsigned char temp;
                                                // Adapte pour CVAVR
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  TWI_msgSize = msgSize;                        // Number of data to transmit.
  TWI_buf[0]  = msg[0];                         // Store slave address with R/W setting.
  if (!( msg[0] & 0b00000001 ))       		// (!( msg[0] & (TRUE<<TWI_READ_BIT) ))
  {						// If it is a write operation, then also copy data.
    for ( temp = 1; temp < msgSize; temp++ )
      TWI_buf[ temp ] = msg[ temp ];
  }
  TWI_statusReg.all = 0;      
  TWI_state         = TWI_NO_STATE ;            //  TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR =  0b10100101;                           // (1<<TWEN)| TWI Interface enabled.
                           			// (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag.
                				// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|Initiate a START condition.
                                      		// (0<<TWWC);
}   
