/********************************************************************
This program was produced by the
CodeWizardAVR V1.24.0 Standard
Automatic Program Generator
 Copyright 1998-2003 HP InfoTech s.r.l.
http://www.hpinfotech.ro
e-mail:office@hpinfotech.ro

Project : PPE Robot solaire
Version : 1
Date    : 28/11/2009
Author  : Philippe Mariano                            
Company : Lyce Pierre Emile Martin 18000 BOURGES                                
Comments: Fichier de la fiche guide 1bis
Mise en oeuvre du protocole I2C par l'interface TWI 
Commande d'un afficheur LCD Batron I2C (PCF2119x)
Lecture des BP et recopie sur les LED

File : twi2bBat.c
Target:  Robot S_ISI
Chip type           : ATmega8535
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 128
*********************************************************************/

// *****************************************************************************************************************************
// Bibliothques et dfinitions
// -----------------------------------------------------------------------------------------------------------------------------
#include <mega8535.h> 
#include <stdlib.h>
#include <string.h>				// Fonction de traitement des chanes de caractres  
#include <delay.h>				// Fonction temporisation  
// -----------------------------------------------------------------------------------------------------------------------------

// Redfinition du buffer TWI dclar dans le fichier twi_master_cvavr.h et appel des fonctions TWI
// -----------------------------------------------------------------------------------------------------------------------------
#define TWI_BUFFER_SIZE   20 			// A placer avant l'appelle de TWI_F.c sinon TWI_BUFFER_SIZE=4 !!!!!!!!!
	
#include "TWI_F.c" 				// Fonctions du driver TWI Master (adaptation de la note ATMEL AVR315 -> CVAVR) 
/***********************************************************************
Project : PPE Robot(s)
Version : 1
Date    : 12/08/2009
Author  : MNO
File : twi_f.c
Description : Fonctions du drivers TWI (I2C), dveloppes par ATMEL
et dcrites dans la note d'application AVR315, adaptes au
cross compilateur CVAVR.
Fichier  inclure en en tte d'une application ncessitant une interface
2 fil I2C.
Evolution:  remplacer par une bibliothque
***********************************************************************/


#include <TWI_Master_CVAVR.h>  

// Declare your global variables here 
//-----------------------------------------------------------------------------------------------------------------------------
static unsigned char TWI_msgSize;             		// Number of bytes to be transmitted.       
static unsigned char TWI_buf[TWI_BUFFER_SIZE];    	// Transceiver buffer
static unsigned char TWI_state = TWI_NO_STATE;      // State byte. Default set to TWI_NO_STATE 
 
union TWI_STR                      					// Status byte holding flags.
{
    unsigned char all;
    unsigned char lastTransOK:1;      
    unsigned char unusedBits:7; 
} TWI_statusReg ={0};  
//-----------------------------------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------------------------------
// Fonctions du drivers TWI
//-----------------------------------------------------------------------------------------------------------------------------
void TWI_Master_Initialise(void)
{                                               // Adapte pour CVAVR
  TWBR = TWI_TWBR;                              // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
  TWDR = 0xFF;                                  // Default content = SDA released.
  												// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR = 0b00000100;                            // (1<<TWEN)| Enable TWI-interface and release TWI pins.
                               					// (0<<TWIE)|(0<<TWINT)|Disable Interupt.
                               					// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests.
                                         		// 0<<TWWC); 
} 
//-----------------------------------------------------------------------------------------------------------------------------
unsigned char TWI_Transceiver_Busy(void)
{                                               // Adapte pour CVAVR
  return ( TWCR & 0b00000001);                  // ( TWCR & (1<<TWIE) ) IF TWI Interrupt is enabled then the Transceiver is busy
}
//-----------------------------------------------------------------------------------------------------------------------------
void TWI_Start_Transc_With_Data( unsigned char *msg, unsigned char msgSize )
{
  unsigned char temp;
                                                // Adapte pour CVAVR
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  TWI_msgSize = msgSize;                        // Number of data to transmit.
  TWI_buf[0]  = msg[0];                         // Store slave address with R/W setting.
  if (!( msg[0] & 0b00000001 ))       			// (!( msg[0] & (TRUE<<TWI_READ_BIT) ))
  {												// If it is a write operation, then also copy data.
    for ( temp = 1; temp < msgSize; temp++ )
      TWI_buf[ temp ] = msg[ temp ];
  }
  TWI_statusReg.all = 0;                        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  TWI_state         = TWI_NO_STATE ;            //  TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR =  0b10100101;                           // (1<<TWEN)| TWI Interface enabled.
                           						// (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag.
                								// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|Initiate a START condition!!!!!.
                                      			// (0<<TWWC);
} 
//-----------------------------------------------------------------------------------------------------------------------------
unsigned char TWI_Get_Data_From_Transceiver( unsigned char *msg, unsigned char msgSize )
{
  unsigned char i;
          
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  if( TWI_statusReg.lastTransOK )               // Last transmission competed successfully.              
  {                                             
    for ( i=0; i<msgSize; i++ )                 // Copy data from Transceiver buffer.
    {
      msg[ i ] = TWI_buf[ i ];
    }
  }
  return( TWI_statusReg.lastTransOK );                                   
}  
//-----------------------------------------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------------------------------------------------------
// Interruption TWI
// ----------------------------------------------------------------------------------------------------------------------------
interrupt [TWI] void twi_isr(void)      		// Adapte pour CVAVR
{												// #pragma vector=TWI_vect
												// __interrupt void TWI_ISR(void)
  static unsigned char TWI_bufPtr;
  
  switch (TWSR)
  {
    case TWI_START:             				// START has been transmitted  
    
    case TWI_REP_START:         				// Repeated START has been transmitted
      TWI_bufPtr = 0;   						// Set buffer pointer to the TWI Address location
      				
    case TWI_MTX_ADR_ACK:       				// SLA+W has been tramsmitted and ACK received 
    
    case TWI_MTX_DATA_ACK:      				// Data byte has been tramsmitted and ACK received
      if (TWI_bufPtr < TWI_msgSize)
      {
        TWDR = TWI_buf[TWI_bufPtr++];           // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag to send byte
                         				  		// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| 
               						  			// (0<<TWWC); 
      }
      else                    					// Send STOP after last byte
      {
        TWI_statusReg.lastTransOK = TRUE;       // Set status bits to completed successfully.
         						  				// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE 
        TWCR = 0b10010100; 		                // (1<<TWEN)| TWI Interface enabled
        										// (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                         				  		// (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                               				  	// (0<<TWWC);
      }
    break;
      
    case TWI_MRX_DATA_ACK:      				// Data byte has been received and ACK tramsmitted
      TWI_buf[TWI_bufPtr++] = TWDR; 
      
    case TWI_MRX_ADR_ACK:       				// SLA+R has been tramsmitted and ACK received
      if (TWI_bufPtr < (TWI_msgSize-1) )        // Detect the last byte to NACK it.
      {                                         // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b11000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  		// (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|Send ACK after reception
                                  				// (0<<TWWC);  
      } else                    				// Send NACK after next reception
      {						          			// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  		// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| Send NACK after reception
                                			  	// (0<<TWWC); 
      }    
    break; 
      
    case TWI_MRX_DATA_NACK:     				// Data byte has been received and NACK tramsmitted
      TWI_buf[TWI_bufPtr] = TWDR;
      TWI_statusReg.lastTransOK = TRUE;         // Set status bits to completed successfully. 
      											// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10010100;                        // (1<<TWEN)| TWI Interface enabled
                                  				// (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                       							// (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                              					// (0<<TWWC); 
    break;  
          
    case TWI_ARB_LOST:          				// Arbitration lost                             
    											// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10100101;                        // (1<<TWEN)| TWI Interface enabled
                                   				// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag
                        						// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)| Initiate a (RE)START condition.
                              					// (0<<TWWC);
    break;
      
    case TWI_MTX_ADR_NACK:      				// SLA+W has been tramsmitted and NACK received
    case TWI_MRX_ADR_NACK:      				// SLA+R has been tramsmitted and NACK received    
    case TWI_MTX_DATA_NACK:     				// Data byte has been tramsmitted and NACK received
//    case TWI_NO_STATE              			// No relevant state information available; TWINT = 0
    case TWI_BUS_ERROR:         				// Bus error due to an illegal START or STOP condition  
    
    default:     
      TWI_state = TWSR;                         // Store TWSR and automatically sets clears noErrors bit.
      											// Reset TWI Interface
                 								// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b00000100;                        // (1<<TWEN)| Enable TWI-interface and release TWI pins
                                   				// (0<<TWIE)|(0<<TWINT)| Disable Interupt
                       							// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests
                              					// (0<<TWWC);
  }
} 

// *****************************************************************************************************************************   

// *****************************************************************************************************************************
// Constantes en flash
// -----------------------------------------------------------------------------------------------------------------------------
// Description des commandes d'initialisation de l'afficheur LCD I2C BATRON (voir doc PCF2119x p22  p25)
/* -----------------------------------------------------------------------------------------------------------------------------
Ordre   Dsignation					Valeur		Remarques
// -----------------------------------------------------------------------------------------------------------------------------
1 	Adresse de l'afficheur 			=	0x76 		(SA0 = 1) [ envoyer en premier!!!]
2 	Select function setting			=	0x00 		Doit prcder une ou plusieurs commandes dans la trame 
									I2C ! (accs au reg IR ) doc PCF2119x p22
3 	Command : Function set			=	0x34            2 lignes de 16 caractres 
4 	Command : Display control		=	0x0c            Display on, curseur off
5 	Command : Entry mode set		=       0x06  		Incrment, no shift
6	Command : Extended instruction set	= 	0x35            -------------------------------------------------------
7	Command : Set display configuration	= 	0x04  		left-to-right, top to bottom 
8	Command : Set coefficient temperature	= 	0x10            TC1 = 0, TC2 = 0 (Par default)
9	Command : Set Internal HVgen stages	=	0x42            HV stage 3  ???
10	Command : Set VLCD			= 	0x9F            set VLCD, store VA (Rglage contraste)
11	Command : Normal instruction set	= 	0x34            -------------------------------------------------------
12	Command : Set DDRAM address		= 	0x80            DDRAM address = 0x00 
// -----------------------------------------------------------------------------------------------------------------------------
Autres commandes
 	Command : Clear display			= 	0x01 
 	Command : Write to DDRAM		=	0x40		Doit prcder une ou plusieurs donnes dans la trame 
 									I2C ! (accs au registre DR)  
-----------------------------------------------------------------------------------------------------------------------------*/ 
// Messages textuels prdfinis 
// ----------------------------------------------------------------------------------------------------------------------------- 
flash char Texte_Accueil_Ligne1[] = "PPE Robsol";
flash char Texte_Accueil_Ligne2[] = "TWI2b.c";

// Message d'initialisation de l'afficheur LCD I2C BATRON (voir doc PCF2119x p22  p25)
// ----------------------------------------------------------------------------------------------------------------------------- 
flash unsigned char Frame_I2C_LCD_BATRON_Init[]={0x76,0x00,0x34,0x0d,0x06,0x35,0x04,0x10,0x42,0x9F,0x34,0x83,0x02};   

// *****************************************************************************************************************************

// *****************************************************************************************************************************
// Prototypes des fonctions
// ----------------------------------------------------------------------------------------------------------------------------- 
void LCD_I2C_BATRON_gotoxy(unsigned char x, unsigned char y);  // x=position dans la ligne [0,15], y=n de la ligne [0,1] 
// Construction d'une trame de texte pour l'afficheur LCD BATRON
void Built_Frame_String_I2C(unsigned char FrameI2C, unsigned char Tstring, int Tstringsize); 
// ***************************************************************************************************************************** 
 

// *****************************************************************************************************************************
// *****************************************************************************************************************************
// Programme principal
// *****************************************************************************************************************************
void main(void)
{
// *****************************************************************************************************************************
// ---------------------------------------------- Declare your local variables here -------------------------------------------- 
// =============================================================================================================================
// Messages pour les composants I2C 
// =============================================================================================================================
// 1) Message pour le buffer TWI
// ----------------------------------------------------------------------------------------------------------------------------- 
// Type 		Nom 				       Description 
// -----------------------------------------------------------------------------------------------------------------------------    
unsigned char Frame_I2C_For_TWI_Buffer[TWI_BUFFER_SIZE];     	// La taille de ce buffer doit tre rgl en fonction de
								// la taille maximum des messages
// ============================================================================================================================= 

// -----------------------------------------------------------------------------------------------------------------------------
// 2) Messages prdfinis pour les ports I2C PCF8574 associs aux BP et aux LEDs de la carte S_ISI
// ----------------------------------------------------------------------------------------------------------------------------- 
// Type 		Nom 				       Description 
// ----------------------------------------------------------------------------------------------------------------------------- 
unsigned char Frame_I2C_LED_All_Off[]={0x40,0xff}; 	// Trame d'extinction des LED {addLed + W (ox40), data = all Led off}
unsigned char Frame_I2C_LED_Switching[]={0x40,0x00}; 	// Trame de commande des LED {addLed + W (0x40), data = Led on=0
							// or off=1}. Initialisation = All on
unsigned char Frame_I2C_BP_Read[]={0x43,0x00}; 		// Trame de lecture des BP {addBP + R (0x43), data=nothing}      
// ============================================================================================================================= 

// -----------------------------------------------------------------------------------------------------------------------------
// 3) Messages pour le LCD I2C BATRON de la carte S_ISI
// ----------------------------------------------------------------------------------------------------------------------------- 
// Type 		Nom 				       Description 
// ----------------------------------------------------------------------------------------------------------------------------- 
unsigned char Frame_I2C_LCD_BATRON_Clear[]={0x76,0x00,0x01}; // Effacement de l'cran {addLCD, acces RegInst, cde effacement}
unsigned char Frame_I2C_Lcd_Line[19]={0x76,0x40};      	     // Trame de texte compose de: {addLCD, commande criture en DDRAM,  
						 	     //  Le texte  afficher : 16 caractre max,  /0 }
unsigned char String_to_LCD[17]; 			     // Reoit Le texte  afficher (16 caractres max. + /0
							     // ATTENTION  la position du curseur sur l'afficheur!!!!!!!!) 
// =============================================================================================================================

// =============================================================================================================================
// Divers variables
// ----------------------------------------------------------------------------------------------------------------------------- 
// Type 		Nom 				       Description 
// -----------------------------------------------------------------------------------------------------------------------------    
unsigned char i, Digit; 
unsigned char Digit_to_String[4]={0xa0,0xa0,0xa0,0};   	// Contient la reprsentation textuelle d'un nombre de 3 chiffres
                                                     	// Initialisation avec 3 espaces (0x20+0x80) et /0 
// -----------------------------------------------------------------------------------------------------------------------------
// *****************************************************************************************************************************


// =============================================================================================================================
// ===================================================== Initialisations =======================================================
// =============================================================================================================================
// Initialisation de l'interface TWI en mode Standby
TWI_Master_Initialise(); 

// Global enable interrupts
#asm("sei")			// L'autorisation des interruptions doit se faire aprs l'initialisation de l'interface TWI				

// Extinction des LEDs de la carte S_ISI
TWI_Start_Transc_With_Data(Frame_I2C_LED_All_Off,sizeof(Frame_I2C_LED_All_Off)); 	

// Initialisation de l'afficheur BATRON
for (i=0;i<13;i++)  
{
  Frame_I2C_For_TWI_Buffer[i]=Frame_I2C_LCD_BATRON_Init[i];   // Tableau MI2C_LCD_BATRON_Init en flash !!!!! Utilisation pointeur ???
} 
TWI_Start_Transc_With_Data(Frame_I2C_For_TWI_Buffer,sizeof(Frame_I2C_LCD_BATRON_Init));

//******************************************************************************************************************************
// Message d'accueil
// -----------------------------------------------------------------------------------------------------------------------------
TWI_Start_Transc_With_Data(Frame_I2C_LCD_BATRON_Clear,sizeof(Frame_I2C_LCD_BATRON_Clear));	// Effacement de l'cran 

// Message 1 sur ligne 1
// -----------------------------------------------------------------------------------------------------------------------------
// Positionner le curseur
LCD_I2C_BATRON_gotoxy(3,0); 
// Copier le texte  envoyer dans le tableau String_to_LCD (16 caractres max)  
strcpyf(String_to_LCD, Texte_Accueil_Ligne1); // strcpyf car constante en flash !
// Rajouter le texte prcdent  l'en tte de la trame et le transcoder pour le LCD BATRON
Built_Frame_String_I2C(Frame_I2C_Lcd_Line, String_to_LCD, strlen(String_to_LCD)); // strlen pour des messages textuels termins 
// Transmettre la trame I2C                                                      // par /0
TWI_Start_Transc_With_Data(Frame_I2C_Lcd_Line,strlen(Frame_I2C_Lcd_Line));
// -----------------------------------------------------------------------------------------------------------------------------	
// Message 2 sur ligne 2 
// ----------------------------------------------------------------------------------------------------------------------------- 
LCD_I2C_BATRON_gotoxy(4,1);
strcpyf(String_to_LCD, Texte_Accueil_Ligne2);  
Built_Frame_String_I2C(Frame_I2C_Lcd_Line, String_to_LCD, strlen(String_to_LCD));  
TWI_Start_Transc_With_Data(Frame_I2C_Lcd_Line,strlen(Frame_I2C_Lcd_Line));
//******************************************************************************************************************************

delay_ms(2000);
 
// Construction d'un message et envoi 
// -----------------------------------------------------------------------------------------------------------------------------
TWI_Start_Transc_With_Data(Frame_I2C_LCD_BATRON_Clear,sizeof(Frame_I2C_LCD_BATRON_Clear));	// Effacement de l'cran
Digit = 145;
itoa((int)Digit, Digit_to_String);			// Conversion de la valeur 145 en chane de caractres
Built_Frame_String_I2C(Frame_I2C_Lcd_Line, Digit_to_String, strlen(Digit_to_String)); 
TWI_Start_Transc_With_Data(Frame_I2C_Lcd_Line,strlen(Frame_I2C_Lcd_Line));
// =============================================================================================================================

// =============================================================================================================================
// ================================================ Boucle principale ==========================================================
// =============================================================================================================================
while (1)
      { 
// ----------------------------------  Champ Lecture des donnes ---------------------------------------------------- 
// Lecture de l'tat des BP
    TWI_Start_Transc_With_Data(Frame_I2C_BP_Read,1);      	// Envoi de l'octet d'adresse des BP + bit RW=lecture(0x43)
    TWI_Get_Data_From_Transceiver(Frame_I2C_BP_Read,2);   	// TWI_Get_Data_From_Transceiver(Donnee_BP,2);
          
// ----------------------------------  Champ Traitement -------------------------------------------------------------        
    Frame_I2C_LED_Switching[1] = ~Frame_I2C_BP_Read[1];   	
    
// ----------------------------------  Champ Ecriture des rsultats  ------------------------------------------------
// Ecriture sur les LED                              	
    TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,2);  // Les LED affichent l'tat des BP
    delay_ms(200);					// Lecture des BP toutes les 200ms
    
      }; // Fin while(1)
      
} //Fin main  
// =============================================================================================================================

// =============================================================================================================================
// Fonctions pour le LCD BATRON    
// A mettre dans une bibliothque
// =============================================================================================================================

void LCD_I2C_BATRON_gotoxy(unsigned char posx , unsigned char posy) 
{
 unsigned char LCD_BATRON_XY[] = {0x76,0x00,0x80};
 
 if((posx<16)&&(posy<2))
 {
  switch(posy)
  {
    case 0:LCD_BATRON_XY[2]=posx+0x80; break;
    case 1:LCD_BATRON_XY[2]=posx+0xC0; break;
  } 
 }
  TWI_Start_Transc_With_Data(LCD_BATRON_XY, sizeof(LCD_BATRON_XY));
} 
// =============================================================================================================================

void Built_Frame_String_I2C(unsigned char FrameI2C, unsigned char Tstring, int Tstringsize)
{     
 int i; 
 unsigned char *p_Tstring, *p_FrameI2C;
 p_Tstring = Tstring; p_FrameI2C = FrameI2C + 2; 
 
 for(i=0;i<Tstringsize;i++)                  
 {
  *p_FrameI2C = *p_Tstring + 0x80;
   p_Tstring++; p_FrameI2C++;
 }
 *p_FrameI2C = 0;
}
// =============================================================================================================================
