// =============================================================================================================================
// Partie dclarative du programme
// =============================================================================================================================
// Bibliothques et dfinitions
// =============================================================================================================================
#include <mega8535.h>      
// -----------------------------------------------------------------------------------------------------------------------------
// Bibliothque  rajouter manuellement   
// -----------------------------------------------------------------------------------------------------------------------------
#include <stdlib.h> 
#include <stdio.h>				// Fonction sprintf
#include <math.h>
#include <string.h>				// Fonctions de traitement des chanes de caractres  
#include <delay.h>				// Fonction temporisation   
// -----------------------------------------------------------------------------------------------------------------------------    
// Bibliothque dveloppes pour la carte SISI    
// -----------------------------------------------------------------------------------------------------------------------------
#include "HMC6352.h"			// Dfinition des registres de la boussole Honeywell HMC6352   
#include "SISISOL.h"            // Gestion de l'IHM SISI 
// -----------------------------------------------------------------------------------------------------------------------------
// Redfinition des adresses (lecture / criture) de la boussole HMC6352 (0x42 par dfaut)
// -----------------------------------------------------------------------------------------------------------------------------
#define NEW_HMC6352_SLA_W 	0x44 // Ox42 est utilis par le PCF8574 reli aux LEDs sur l'IHM SISIV1 ! 
#define	NEW_HMC6352_SLA_R   0x45 

#define Empty	0	
// -----------------------------------------------------------------------------------------------------------------------------
// Redfinition du buffer TWI dclar dans le fichier twi_master_cvavr.h et appel des fonctions TWI
// -----------------------------------------------------------------------------------------------------------------------------
#define TWI_BUFFER_SIZE   20 			// A placer avant l'appelle de TWI_F.c sinon TWI_BUFFER_SIZE=4 !!!!!!!!!
#include "TWI_F.c" 			   			// Fonctions du driver TWI Master (adaptation de la note ATMEL AVR315 -> CVAVR) 
/***********************************************************************
Project : PPE Robot(s)
Version : 1
Date    : 12/08/2009
Author  : MNO
File : twi_f.c
Description : Fonctions du drivers TWI (I2C), dveloppes par ATMEL
et dcrites dans la note d'application AVR315, adaptes au
cross compilateur CVAVR.
Fichier  inclure en en tte d'une application ncessitant une interface
2 fil I2C.
Evolution:  remplacer par une bibliothque
***********************************************************************/


#include <TWI_Master_CVAVR.h>  

// Declare your global variables here 
//-----------------------------------------------------------------------------------------------------------------------------
static unsigned char TWI_msgSize;             		// Number of bytes to be transmitted.       
static unsigned char TWI_buf[TWI_BUFFER_SIZE];    	// Transceiver buffer
static unsigned char TWI_state = TWI_NO_STATE;      // State byte. Default set to TWI_NO_STATE 
 
union TWI_STR                      					// Status byte holding flags.
{
    unsigned char all;
    unsigned char lastTransOK:1;      
    unsigned char unusedBits:7; 
} TWI_statusReg ={0};  
//-----------------------------------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------------------------------
// Fonctions du drivers TWI
//-----------------------------------------------------------------------------------------------------------------------------
void TWI_Master_Initialise(void)
{                                               // Adapte pour CVAVR
  TWBR = TWI_TWBR;                              // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
  TWDR = 0xFF;                                  // Default content = SDA released.
  												// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR = 0b00000100;                            // (1<<TWEN)| Enable TWI-interface and release TWI pins.
                               					// (0<<TWIE)|(0<<TWINT)|Disable Interupt.
                               					// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests.
                                         		// 0<<TWWC); 
} 
//-----------------------------------------------------------------------------------------------------------------------------
unsigned char TWI_Transceiver_Busy(void)
{                                               // Adapte pour CVAVR
  return ( TWCR & 0b00000001);                  // ( TWCR & (1<<TWIE) ) IF TWI Interrupt is enabled then the Transceiver is busy
}
//-----------------------------------------------------------------------------------------------------------------------------
void TWI_Start_Transc_With_Data( unsigned char *msg, unsigned char msgSize )
{
  unsigned char temp;
                                                // Adapte pour CVAVR
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  TWI_msgSize = msgSize;                        // Number of data to transmit.
  TWI_buf[0]  = msg[0];                         // Store slave address with R/W setting.
  if (!( msg[0] & 0b00000001 ))       			// (!( msg[0] & (TRUE<<TWI_READ_BIT) ))
  {												// If it is a write operation, then also copy data.
    for ( temp = 1; temp < msgSize; temp++ )
      TWI_buf[ temp ] = msg[ temp ];
  }
  TWI_statusReg.all = 0;                        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  TWI_state         = TWI_NO_STATE ;            //  TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR =  0b10100101;                           // (1<<TWEN)| TWI Interface enabled.
                           						// (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag.
                								// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|Initiate a START condition!!!!!.
                                      			// (0<<TWWC);
} 
//-----------------------------------------------------------------------------------------------------------------------------
unsigned char TWI_Get_Data_From_Transceiver( unsigned char *msg, unsigned char msgSize )
{
  unsigned char i;
          
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  if( TWI_statusReg.lastTransOK )               // Last transmission competed successfully.              
  {                                             
    for ( i=0; i<msgSize; i++ )                 // Copy data from Transceiver buffer.
    {
      msg[ i ] = TWI_buf[ i ];
    }
  }
  return( TWI_statusReg.lastTransOK );                                   
}  
//-----------------------------------------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------------------------------------------------------
// Interruption TWI
// ----------------------------------------------------------------------------------------------------------------------------
interrupt [TWI] void twi_isr(void)      		// Adapte pour CVAVR
{												// #pragma vector=TWI_vect
												// __interrupt void TWI_ISR(void)
  static unsigned char TWI_bufPtr;
  
  switch (TWSR)
  {
    case TWI_START:             				// START has been transmitted  
    
    case TWI_REP_START:         				// Repeated START has been transmitted
      TWI_bufPtr = 0;   						// Set buffer pointer to the TWI Address location
      				
    case TWI_MTX_ADR_ACK:       				// SLA+W has been tramsmitted and ACK received 
    
    case TWI_MTX_DATA_ACK:      				// Data byte has been tramsmitted and ACK received
      if (TWI_bufPtr < TWI_msgSize)
      {
        TWDR = TWI_buf[TWI_bufPtr++];           // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag to send byte
                         				  		// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| 
               						  			// (0<<TWWC); 
      }
      else                    					// Send STOP after last byte
      {
        TWI_statusReg.lastTransOK = TRUE;       // Set status bits to completed successfully.
         						  				// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE 
        TWCR = 0b10010100; 		                // (1<<TWEN)| TWI Interface enabled
        										// (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                         				  		// (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                               				  	// (0<<TWWC);
      }
    break;
      
    case TWI_MRX_DATA_ACK:      				// Data byte has been received and ACK tramsmitted
      TWI_buf[TWI_bufPtr++] = TWDR; 
      
    case TWI_MRX_ADR_ACK:       				// SLA+R has been tramsmitted and ACK received
      if (TWI_bufPtr < (TWI_msgSize-1) )        // Detect the last byte to NACK it.
      {                                         // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b11000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  		// (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|Send ACK after reception
                                  				// (0<<TWWC);  
      } else                    				// Send NACK after next reception
      {						          			// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  		// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| Send NACK after reception
                                			  	// (0<<TWWC); 
      }    
    break; 
      
    case TWI_MRX_DATA_NACK:     				// Data byte has been received and NACK tramsmitted
      TWI_buf[TWI_bufPtr] = TWDR;
      TWI_statusReg.lastTransOK = TRUE;         // Set status bits to completed successfully. 
      											// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10010100;                        // (1<<TWEN)| TWI Interface enabled
                                  				// (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                       							// (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                              					// (0<<TWWC); 
    break;  
          
    case TWI_ARB_LOST:          				// Arbitration lost                             
    											// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10100101;                        // (1<<TWEN)| TWI Interface enabled
                                   				// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag
                        						// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)| Initiate a (RE)START condition.
                              					// (0<<TWWC);
    break;
      
    case TWI_MTX_ADR_NACK:      				// SLA+W has been tramsmitted and NACK received
    case TWI_MRX_ADR_NACK:      				// SLA+R has been tramsmitted and NACK received    
    case TWI_MTX_DATA_NACK:     				// Data byte has been tramsmitted and NACK received
//    case TWI_NO_STATE              			// No relevant state information available; TWINT = 0
    case TWI_BUS_ERROR:         				// Bus error due to an illegal START or STOP condition  
    
    default:     
      TWI_state = TWSR;                         // Store TWSR and automatically sets clears noErrors bit.
      											// Reset TWI Interface
                 								// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b00000100;                        // (1<<TWEN)| Enable TWI-interface and release TWI pins
                                   				// (0<<TWIE)|(0<<TWINT)| Disable Interupt
                       							// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests
                              					// (0<<TWWC);
  }
} 

										// A remplacer par une bibliothque ! 
										
void main(void)
{
// =============================================================================================================================
// ---------------------------------------------- Declare your local variables here -------------------------------------------- 
// =============================================================================================================================
// Messages pour les composants I2C 
// =============================================================================================================================
// 1) Messages prdfinis pour les ports I2C PCF8574 associs aux BP et aux LEDs de la carte S_ISI
// ----------------------------------------------------------------------------------------------------------------------------- 
// Type 	   		Nom 				       						Description 
// ----------------------------------------------------------------------------------------------------------------------------- 
// =============================================================================================================================
// 4) Liste des messages prdfinis pour la boussole HMC6352
// =============================================================================================================================
// Lecture de la direction (comprise entre 0 et 3599, code sur deux octets)
unsigned char Frame_I2C_W_Cde_HMC6352[] = {NEW_HMC6352_SLA_W,'A'}; 
unsigned char Frame_I2C_Read_Data_HMC6352[] = {NEW_HMC6352_SLA_R,Empty,Empty}; 

// Lecture Adresse base
unsigned char Frame_I2C_W_Add_EEPROM_HMC6352[] = {NEW_HMC6352_SLA_W,'r',Operational_Mode_EEPROM};   
unsigned char Frame_I2C_Read_Add_Reg_HMC6352[] = {NEW_HMC6352_SLA_R,Empty};
 
// Trame de changement de l'addresse de la boussole (A faire une seule fois)
unsigned char Frame_I2C_Change_Add_HMC6352[] = {HMC6352_SLA_W,'w',HMC6352_Reg_ADD_BASE,NEW_HMC6352_SLA_W}; 

// =============================================================================================================================
// ===================================================== Initialisations =======================================================
// =============================================================================================================================
TWI_Master_Initialise(); 									// Initialisation de l'interface TWI en mode Standby 
// -----------------------------------------------------------------------------------------------------------------------------
// Global enable interrupts
#asm("sei")							// L'autorisation des interruptions doit se faire aprs l'initialisation de l'interface TWI


// Prparation de l'opration de lecture en EEPROM : Ecriture de la commande 'r' et de l'argument "EEPROM Address"
//TWI_Start_Transc_With_Data(Frame_I2C_W_Add_EEPROM_HMC6352,sizeof(Frame_I2C_W_Add_EEPROM_HMC6352)); // (1)



while(1)
{

// Initialisation de la boussole HMC6352
// ----------------------------------------------------------------------------------------------------------------------------- 
// Lecture de l'adresse EEPROM prcise en (1)
// TWI_Start_Transc_With_Data(Frame_I2C_Read_Add_Reg_HMC6352,sizeof(Frame_I2C_Read_Add_Reg_HMC6352));
//Prparation de l'opration de lecture des donnes: Ecriture de la commande 'A' 
TWI_Start_Transc_With_Data(Frame_I2C_W_Cde_HMC6352, sizeof(Frame_I2C_W_Cde_HMC6352));
TWI_Start_Transc_With_Data(Frame_I2C_Read_Data_HMC6352,sizeof(Frame_I2C_Read_Data_HMC6352));
delay_us(2000); // pour affichage stable sur l'oscilloscope

// -----------------------------------------------------------------------------------------------------------------------------
}
}
