/********************************************************************
This program was produced by the
CodeWizardAVR V1.24.0 Standard
Automatic Program Generator
 Copyright 1998-2003 HP InfoTech s.r.l.
http://www.hpinfotech.ro
e-mail:office@hpinfotech.ro

Project : ROBOT BIPED
Version : 1.0.1
Date    : 13/06/2010
Mise  jour: 17/06/2010
Author  : Philippe Mariano                            
Company : Lyce Pierre Emile Martin 18000 BOURGES                                
Comments: 
Mise en oeuvre du protocole I2C par l'interface TWI 
- Commande des LEDs. 
- Test de la carte SD21 (Commande de quatre servomoteurs
plac aux positions 1, .., 4 de la carte

File : twi5SD21.c
Target: Cartes SISI et SD21  
Chip type           : ATmega8535
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 128
*********************************************************************/

// =============================================================================================================================
// =============================================================================================================================
// Partie dclarative du programme
// =============================================================================================================================
// Bibliothques et dfinitions
// =============================================================================================================================
#include <mega8535.h>      
// -----------------------------------------------------------------------------------------------------------------------------
// Bibliothque  rajouter manuellement   
// -----------------------------------------------------------------------------------------------------------------------------
#include <stdlib.h>  
#include <delay.h>				// Fonctions de temporisation   
// -----------------------------------------------------------------------------------------------------------------------------    
// Bibliothque dveloppes pour la carte SISI  placer dans le rpertoire du projet   
// -----------------------------------------------------------------------------------------------------------------------------
#include "SD21.h"				// Dfinitions des registres de la carte servo SD21   
#include "SISISOL.h"            // Gestion de l'IHM SISI  
// -----------------------------------------------------------------------------------------------------------------------------
// Alias pour la trame de commande des Servos 
// -----------------------------------------------------------------------------------------------------------------------------
#define NumServo			  Frame_I2C_Servo[1]
#define RegSpeedValue	      Frame_I2C_Servo[2] 
#define RegPosLByte			  Frame_I2C_Servo[3] 
#define RegPosHByte			  Frame_I2C_Servo[4]
// -----------------------------------------------------------------------------------------------------------------------------
// Redfinition du buffer TWI dclar dans le fichier twi_master_cvavr.h et appel des fonctions TWI
// -----------------------------------------------------------------------------------------------------------------------------
#define TWI_BUFFER_SIZE   		20 		// A placer avant l'appelle de TWI_F.c sinon TWI_BUFFER_SIZE=4 !!!!!!!!!
#include "TWI_F.c" 			   			// Fonctions du driver TWI Master (adaptation de la note ATMEL AVR315 -> CVAVR) 
/***********************************************************************
Project : PPE Robot(s)
Version : 1
Date    : 12/08/2009
Author  : MNO
File : twi_f.c
Description : Fonctions du drivers TWI (I2C), dveloppes par ATMEL
et dcrites dans la note d'application AVR315, adaptes au
cross compilateur CVAVR.
Fichier  inclure en en tte d'une application ncessitant une interface
2 fil I2C.
Evolution:  remplacer par une bibliothque
***********************************************************************/


#include <TWI_Master_CVAVR.h>  

// Declare your global variables here 
//-----------------------------------------------------------------------------------------------------------------------------
static unsigned char TWI_msgSize;             		// Number of bytes to be transmitted.       
static unsigned char TWI_buf[TWI_BUFFER_SIZE];    	// Transceiver buffer
static unsigned char TWI_state = TWI_NO_STATE;      // State byte. Default set to TWI_NO_STATE 
 
union TWI_STR                      					// Status byte holding flags.
{
    unsigned char all;
    unsigned char lastTransOK:1;      
    unsigned char unusedBits:7; 
} TWI_statusReg ={0};  
//-----------------------------------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------------------------------
// Fonctions du drivers TWI
//-----------------------------------------------------------------------------------------------------------------------------
void TWI_Master_Initialise(void)
{                                               // Adapte pour CVAVR
  TWBR = TWI_TWBR;                              // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
  TWDR = 0xFF;                                  // Default content = SDA released.
  												// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR = 0b00000100;                            // (1<<TWEN)| Enable TWI-interface and release TWI pins.
                               					// (0<<TWIE)|(0<<TWINT)|Disable Interupt.
                               					// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests.
                                         		// 0<<TWWC); 
} 
//-----------------------------------------------------------------------------------------------------------------------------
unsigned char TWI_Transceiver_Busy(void)
{                                               // Adapte pour CVAVR
  return ( TWCR & 0b00000001);                  // ( TWCR & (1<<TWIE) ) IF TWI Interrupt is enabled then the Transceiver is busy
}
//-----------------------------------------------------------------------------------------------------------------------------
void TWI_Start_Transc_With_Data( unsigned char *msg, unsigned char msgSize )
{
  unsigned char temp;
                                                // Adapte pour CVAVR
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  TWI_msgSize = msgSize;                        // Number of data to transmit.
  TWI_buf[0]  = msg[0];                         // Store slave address with R/W setting.
  if (!( msg[0] & 0b00000001 ))       			// (!( msg[0] & (TRUE<<TWI_READ_BIT) ))
  {												// If it is a write operation, then also copy data.
    for ( temp = 1; temp < msgSize; temp++ )
      TWI_buf[ temp ] = msg[ temp ];
  }
  TWI_statusReg.all = 0;                        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  TWI_state         = TWI_NO_STATE ;            //  TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR =  0b10100101;                           // (1<<TWEN)| TWI Interface enabled.
                           						// (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag.
                								// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|Initiate a START condition!!!!!.
                                      			// (0<<TWWC);
} 
//-----------------------------------------------------------------------------------------------------------------------------
unsigned char TWI_Get_Data_From_Transceiver( unsigned char *msg, unsigned char msgSize )
{
  unsigned char i;
          
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  if( TWI_statusReg.lastTransOK )               // Last transmission competed successfully.              
  {                                             
    for ( i=0; i<msgSize; i++ )                 // Copy data from Transceiver buffer.
    {
      msg[ i ] = TWI_buf[ i ];
    }
  }
  return( TWI_statusReg.lastTransOK );                                   
}  
//-----------------------------------------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------------------------------------------------------
// Interruption TWI
// ----------------------------------------------------------------------------------------------------------------------------
interrupt [TWI] void twi_isr(void)      		// Adapte pour CVAVR
{												// #pragma vector=TWI_vect
												// __interrupt void TWI_ISR(void)
  static unsigned char TWI_bufPtr;
  
  switch (TWSR)
  {
    case TWI_START:             				// START has been transmitted  
    
    case TWI_REP_START:         				// Repeated START has been transmitted
      TWI_bufPtr = 0;   						// Set buffer pointer to the TWI Address location
      				
    case TWI_MTX_ADR_ACK:       				// SLA+W has been tramsmitted and ACK received 
    
    case TWI_MTX_DATA_ACK:      				// Data byte has been tramsmitted and ACK received
      if (TWI_bufPtr < TWI_msgSize)
      {
        TWDR = TWI_buf[TWI_bufPtr++];           // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag to send byte
                         				  		// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| 
               						  			// (0<<TWWC); 
      }
      else                    					// Send STOP after last byte
      {
        TWI_statusReg.lastTransOK = TRUE;       // Set status bits to completed successfully.
         						  				// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE 
        TWCR = 0b10010100; 		                // (1<<TWEN)| TWI Interface enabled
        										// (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                         				  		// (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                               				  	// (0<<TWWC);
      }
    break;
      
    case TWI_MRX_DATA_ACK:      				// Data byte has been received and ACK tramsmitted
      TWI_buf[TWI_bufPtr++] = TWDR; 
      
    case TWI_MRX_ADR_ACK:       				// SLA+R has been tramsmitted and ACK received
      if (TWI_bufPtr < (TWI_msgSize-1) )        // Detect the last byte to NACK it.
      {                                         // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b11000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  		// (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|Send ACK after reception
                                  				// (0<<TWWC);  
      } else                    				// Send NACK after next reception
      {						          			// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                      // (1<<TWEN)| TWI Interface enabled
        										// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  		// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| Send NACK after reception
                                			  	// (0<<TWWC); 
      }    
    break; 
      
    case TWI_MRX_DATA_NACK:     				// Data byte has been received and NACK tramsmitted
      TWI_buf[TWI_bufPtr] = TWDR;
      TWI_statusReg.lastTransOK = TRUE;         // Set status bits to completed successfully. 
      											// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10010100;                        // (1<<TWEN)| TWI Interface enabled
                                  				// (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                       							// (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                              					// (0<<TWWC); 
    break;  
          
    case TWI_ARB_LOST:          				// Arbitration lost                             
    											// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10100101;                        // (1<<TWEN)| TWI Interface enabled
                                   				// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag
                        						// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)| Initiate a (RE)START condition.
                              					// (0<<TWWC);
    break;
      
    case TWI_MTX_ADR_NACK:      				// SLA+W has been tramsmitted and NACK received
    case TWI_MRX_ADR_NACK:      				// SLA+R has been tramsmitted and NACK received    
    case TWI_MTX_DATA_NACK:     				// Data byte has been tramsmitted and NACK received
//    case TWI_NO_STATE              			// No relevant state information available; TWINT = 0
    case TWI_BUS_ERROR:         				// Bus error due to an illegal START or STOP condition  
    
    default:     
      TWI_state = TWSR;                         // Store TWSR and automatically sets clears noErrors bit.
      											// Reset TWI Interface
                 								// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b00000100;                        // (1<<TWEN)| Enable TWI-interface and release TWI pins
                                   				// (0<<TWIE)|(0<<TWINT)| Disable Interupt
                       							// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests
                              					// (0<<TWWC);
  }
} 

										// A faire vomuer vers une bibliothque !  
// =============================================================================================================================
#include "Table_Rob.c"                 // Table des valeurs des positions  envoyer aux servos
/***********************************************************************
Project : Robot(s) biped
Version : 1
Date    : 17/06/2010
Author  : Mariano - LEGENDRE
File : Table_Rob.c
Description : Table contenant les valeurs des positions 
 envoyer aux servos
***********************************************************************/


// =============================================================================================================================
// ---------------------------------------------- Declare your local variables here -------------------------------------------- 
// =============================================================================================================================
// Tables des valeurs "Servox" et "Position" places dans les trames Frame_I2C_Servo.
// Chaque table doit avoir le mme nombre de valeurs (max 65536)

flash unsigned char Tab_Servo[]={
Servo1, 	/* N du servo dans la premire trame */
Servo2, 	/* N du servo dans la deuxime trame */
Servo3, 
Servo4 
/*  complter ... */
};  

flash unsigned int Tab_Position[]={
800, 	/* Consigne de position du servo dans la premire trame */
1000,	/* Consigne de position du servo dans la deuxime trame */
1500,
2000
/*  complter ... */
};
// =============================================================================================================================
// =============================================================================================================================
// Fonctions dveloppes pour l'application
// -----------------------------------------------------------------------------------------------------------------------------
// Rcupration de l'octet de poids faible d'un entier
unsigned char LSBYTE(int value)
{
	unsigned int LByte;
	LByte = (unsigned char)(value & 0x00FF);
	return LByte;
} 
// -----------------------------------------------------------------------------------------------------------------------------
// Rcupration de l'octet de poids fort d'un entier
unsigned char MSBYTE(int value)
{
	unsigned int MByte;
	MByte = (unsigned char)((value & 0xFF00)>>8);
	return MByte;
}  
// =============================================================================================================================  


// =============================================================================================================================
// =============================================================================================================================
// Partie excutive du programme 
// =============================================================================================================================
// =============================================================================================================================
void main(void)
{
// =============================================================================================================================
// ---------------------------------------------- Declare your local variables here -------------------------------------------- 
// =============================================================================================================================
unsigned char Vitesse;				// Vitesse de dplacement du servo          
unsigned int tempo;                 // Attente entre deux transmission : doit tre > 20ms 
unsigned int i;						// Variable de boucle 

// -----------------------------------------------------------------------------------------------------------------------------
// Messages pour les composants I2C de la carte SISI
// =============================================================================================================================
// 1) Messages prdfinis pour les ports I2C PCF8574 associs aux BP et aux LEDs de la carte S_ISI
// ----------------------------------------------------------------------------------------------------------------------------- 
// Type 	   		Nom 				       						Description 
// ----------------------------------------------------------------------------------------------------------------------------- 
unsigned char Frame_I2C_LED_Switching[]={Add_LED_I2C,0x00}; 	// Trame de commande des LEDs {SLA+W=0x40, Data : Led_On=0
																// or Off=1}. Initialisation = All On.  
// ============================================================================================================================= 
// 2) Messages pour le LCD I2C BATRON de la carte S_ISI
// ----------------------------------------------------------------------------------------------------------------------------- 
// Type 	   		Nom 				       						Description 
// ----------------------------------------------------------------------------------------------------------------------------- 
// =============================================================================================================================
// 3) Messages prdfinis pour la carte SD21 
// ============================================================================================================================= 
// 30) Initialisation 
// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// 31) Trame de commande des servos
// ----------------------------------------------------------------------------------------------------------------------------- 
// Cette trame est compose de: {@SD21, NumServo, RegSpeedValue, RegPosLByte, RegPosHByte}
unsigned char Frame_I2C_Servo[]={SD21_ADD_BASE_W,Servo1,0,0xE8,0x03};		 

// Dtail du contenu de la "trame I2C Servo" 
// @SD21			  SLA+W = 0xC2: Adresse de la carte SD21 en criture
// NumServo			  N du registre vitesse du servo  commander, dfinit dans SD21.h (Servo1 par dfaut)
// RegSpeedValue	  Valeur  placer dans le le registre vitesse (Vitesse maxi par dfaut)
// RegPosLByte		  Octet de poids faible du mot de commande de la position (Position mini par dfaut)
// RegPosHByte		  Octet de poids fort du mot de commande de la position 
// -----------------------------------------------------------------------------------------------------------------------------
// 32) Lecture des registres de la carte SD21
// -----------------------------------------------------------------------------------------------------------------------------
// ============================================================================================================================= 
// =============================================================================================================================
// Variables diverses
// ----------------------------------------------------------------------------------------------------------------------------- 
// Type 		Nom 				       Description 
// -----------------------------------------------------------------------------------------------------------------------------    
// 1) Gestion de la carte SD21
// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// 2) Gestion du LCD BATRON de la carte Sisi
// -----------------------------------------------------------------------------------------------------------------------------
 // ----------------------------------------------------------------------------------------------------------------------------- 
// 3) Gestion des BP de la carte Sisi
// -----------------------------------------------------------------------------------------------------------------------------
// *****************************************************************************************************************************

// =============================================================================================================================
// ===================================================== Initialisations =======================================================
// =============================================================================================================================
TWI_Master_Initialise(); 			// Initialisation de l'interface TWI en mode Standby                                        
// -----------------------------------------------------------------------------------------------------------------------------
// Global enable interrupts
#asm("sei")							// L'autorisation des interruptions doit se faire aprs l'initialisation de l'interface TWI			
// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// Initialisation de l'afficheur I2C BATRON 
// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// Affichage d'un message d'accueil
// -----------------------------------------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// Initialisations des paramettres de la marche du robot !!!!!!!!!!!!!!!!!!!!
// ----------------------------------------------------------------------------------------------------------------------------- 
Vitesse = 0; 		// Vitesse servo = max		
tempo = 1000;
// Initialisation des registres de la carte SD21
// -----------------------------------------------------------------------------------------------------------------------------
// Position initial du servo1
RegSpeedValue = Vitesse; RegPosLByte= 0xE8; RegPosHByte=0x03;  					// 1000s = 03E8h
TWI_Start_Transc_With_Data(Frame_I2C_Servo,sizeof(Frame_I2C_Servo));			// Vitesse, LSByte, MSByte                           
delay_ms(5000);                                                                                                                
// =============================================================================================================================
// ================================================ Boucle principale ==========================================================
// ============================================================================================================================= 
// Pour info
// 1000 = 3E8h	valeur min
// 1250 = 4E2h
// 1500 = 5DCh
// 1750 = 6D6h 
// 1950 = 7E9h et + ?
// -----------------------------------------------------------------------------------------------------------------------------
// Les valeurs n Servo + Position transmises  la carte SD21 par le bus I2C  doivent tre places dans les tableaux situs
// dans le fichier "Table_Rob.c"
// Le nom des servos est dclar dans le fichier sd21.h
 
while (1)
  { 
   // Configuration des paramtres "Servox" [0<x<5] et "position" envoys aux servos 
   for(i=0; i<sizeof(Tab_Servo); i++)
   {
    // Configuration de la trame "Frame_I2C_Servo" avec les valeurs contenues dans les tableaux situs dans 
    // le fichier Table_Rob.c
    NumServo = Tab_Servo[i]; RegPosLByte = LSBYTE(Tab_Position[i]);  RegPosHByte = MSBYTE(Tab_Position[i]); 
     	
	// Envoi de la trame  "Frame_I2C_Servo"  la carte SD21
	TWI_Start_Transc_With_Data(Frame_I2C_Servo,sizeof(Frame_I2C_Servo));	      
	
	// Visualisation de la position des valeurs dans les tableaux sur les Leds de la carte Sisi
	Frame_I2C_LED_Switching[1] = i;
	TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,sizeof(Frame_I2C_LED_Switching));
	delay_ms(tempo);   
   } // fin du for      
   
  } // fin du while  
  
} // fin du main

