/****************************************************************
Project : PPE Robot Solaire
Version : 1.0.0
// --------------------------------------------------------------
Fichier source : mainTestSISIV2.c
Date de  cration : 06/01/2010
Mise  jour le : 14/11/2010
// --------------------------------------------------------------
Author  : Philippe Mariano
Company : Lyce Pierre Emile Martin 18000 BOURGES
// --------------------------------------------------------------
Comments:
- Mise en oeuvre du protocole I2C sur l'interface TWI
(Adaptation de la note d'application AVR315 d'ATMEL)
- Commande d'un afficheur LCD Batron I2C (PCF2119x)
- Lecture des BP et recopie sur les LED.
- Test de la carte MD23(Reglage et commande des moteurs en
Mode0, lecture et affichage des registres)
- Test de la carte SD21
// --------------------------------------------------------------
Cross compilateur: CVAVR 2.04.5b
// --------------------------------------------------------------
Chip type           : ATmega8535
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 512
****************************************************************/

// =============================================================================================================================
// Partie dclarative du programme
// =============================================================================================================================
// Bibliothques et dfinitions
// =============================================================================================================================
#include <mega8535.h>
// -----------------------------------------------------------------------------------------------------------------------------
// Bibliothque  rajouter manuellement
// -----------------------------------------------------------------------------------------------------------------------------
#include <stdlib.h>
#include <stdio.h>				// Fonction sprintf
#include <math.h>
#include <string.h>				// Fonctions de traitement des chanes de caractres
#include <delay.h>				// Fonction temporisation
#include "DriverTWICV2.h"       // Fonctions du drivers TWI adaptes pour CVAVR
                                // ATTENTION: Vrifier que la taille du buffer est adapte au message (20 par dfaut)
#include <stdmno.h>             // Fonctions LSBYTE et MSBYTE
// -----------------------------------------------------------------------------------------------------------------------------
// Bibliothque dveloppes pour la carte SISI
// -----------------------------------------------------------------------------------------------------------------------------
#include "MD23.h"				// Dfinitions des registres de la carte moteur MD23
#include "LCDBATRON.h"          // Fonctions de lecture/criture des priphriques I2C
#include "SISISOL.h"            // Gestion de l'IHM SISI
#include "SD21.h"				// Dfinitions des registres de la carte servo SD21
#include "TableRob.h"           // Dclaration des tables de valeurs envoyes aux servos

// -----------------------------------------------------------------------------------------------------------------------------
// Dclarations divers
// -----------------------------------------------------------------------------------------------------------------------------
// Alias pour la trame de commande des Servos connects  la carte SD21
// -----------------------------------------------------------------------------------------------------------------------------
#define NumServo			  Frame_I2C_Servo[1]
#define RegSpeedValue	      Frame_I2C_Servo[2] 
#define RegPosLByte			  Frame_I2C_Servo[3] 
#define RegPosHByte			  Frame_I2C_Servo[4]
// -----------------------------------------------------------------------------------------------------------------------------
// Alias pour la trame de commande des moteurs connects  de la carte MD23 - 25
// -----------------------------------------------------------------------------------------------------------------------------
#define	Moteur1			0
#define	Moteur2			1
// =============================================================================================================================

// =============================================================================================================================
// =============================================================================================================================
// Constantes en flash
// -----------------------------------------------------------------------------------------------------------------------------
//Structure pour l'affichage du nom des registres  (A terminer)
//flash char Table_Name_Register_MD23[3][7]={"Speed1", "Speed2","Enc1a "};   // changer par une structure
// -----------------------------------------------------------------------------------------------------------------------------
// Messages textuels prdfinis
// ATTENTION: imprativement 16 caractres
// -----------------------------------------------------------------------------------------------------------------------------
flash char *Texte_Accueil_Ligne1 	= "   Test SISI    ";
flash char *Texte_Accueil_Ligne2 	= "  [TWI141210]   ";
flash char *TexteMenuLedBp			= "1 Test BP->Led  ";
flash char *TexteMenuMD23			= "2 Test MD23     ";
flash char *TexteMenuSD21			= "3 Test SD21     ";
flash char *Ligne0MenuMode			= "21 Choix Modes  ";
flash char *Ligne0MenuInfos			= "22 Aff Infos    ";
flash char *TexteTestSD21           = "Cde Servo 1->4  ";
flash char *TexteTest1BpsurLed	    = "Recopie Bp->Led ";
flash char *Ligne1PlusMoins			= "[+/-]   [Echap] ";
flash char *Ligne1ChoixEntr			= "[Entr]          ";
flash char *Ligne1ChoixEchap		= "        [Echap] ";
flash char *Ligne1ChoixEchap5s		= "    [Echap 5s!] ";
flash char *Ligne1ChoixEchapOk		= "[Entr]  [Echap] ";
flash char *BpPlusSurD0				= "+ -> D0 [Echap] ";
flash char *BpFlecheHSurD1			= "FH-> D1 [Echap] ";
flash char *BpFlecheHSurD2			= "FB-> D2 [Echap] ";
flash char *BpMoinsSurD3			= "- -> D3 [Echap] ";
flash char *BpSetSurD4				= "SET->D4 [Echap] ";
flash char *BpOkSurD5				= "Ok-> D5 [Echap] ";
flash char *BpEntrSurD6				= "ENT->D6 [Echap] ";
flash char *BpEchSurD7				= "ECH->D7 [Echap] ";

// =============================================================================================================================

// =============================================================================================================================
// =============================================================================================================================
// Prototypes des fonctions en cours de test
// -----------------------------------------------------------------------------------------------------------------------------
// =============================================================================================================================

// =============================================================================================================================
// =============================================================================================================================
// ---------------------------------------------- Declare your global variables here -------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------
// Type				Nom														Description
// -----------------------------------------------------------------------------------------------------------------------------
// Etats du graphe des transitions "Menus"
// -----------------------------------------------------------------------------------------------------------------------------
enum TMenu {Menu1_LED_BP, Menu2_MD23, Menu3_SD21, Copie_BP_Sur_Led, Menu21_Mode_MD23, Menu22_Infos_MD23, Choix_Mode_MD23,
Reglage_Valeurs_Mode0_MD23, Reglage_Mode1_MD23, Reglage_Mode2_MD23,Reglage_Mode3_MD23, Controle_Moteurs, Info_TensionBatt,
Info_Version, Info_All, Test_SD21} Menu=Menu1_LED_BP, Menu_1=Menu2_MD23;
// -----------------------------------------------------------------------------------------------------------------------------
bit Choix_Moteur=0;			// Permet de permuter le rglage de la vitesse des moteurs dans le menu mode 0
// =============================================================================================================================


// =============================================================================================================================
// =============================================================================================================================
// Partie excutive du programme
// =============================================================================================================================
// =============================================================================================================================
void main(void)
{
// =============================================================================================================================
// ---------------------------------------------- Declare your local variables here --------------------------------------------
// =============================================================================================================================
// Messages pour les composants I2C
// =============================================================================================================================
// 1) Messages prdfinis pour les ports I2C PCF8574 associs aux BP et aux LEDs de la carte S_ISI
// -----------------------------------------------------------------------------------------------------------------------------
// Type 	   		Nom 				       						Description
// -----------------------------------------------------------------------------------------------------------------------------
char Frame_I2C_LED_All_Off[]={Add_LED_I2C_W,0xff}; 		// Trame d'extinction des LED {SLA+W=0x40, Data = all Led off}
char Frame_I2C_LED_Switching[]={Add_LED_I2C_W,0x00}; 	// Trame de commande des LED {SLA+W=0x40, Data : Led_On=0
																// or Off=1}. Initialisation = All On.
char Frame_I2C_BP_Read[]={Add_BP_I2C_R,0x00}; 			// Trame de lecture des BP {SLA+R=0x43, Data=Nul}
// =============================================================================================================================
// 2) Messages pour le LCD I2C BATRON de la carte S_ISI
// -----------------------------------------------------------------------------------------------------------------------------
// Type 	   		Nom 				       						Description
// -----------------------------------------------------------------------------------------------------------------------------
//unsigned char Frame_I2C_LCD_BATRON_Clear[]={0x76,0x00,0x01}; 	// Effacement de l'cran {SLA+W=0x76, acces=RegIR, cde=clear}
char Frame_I2C_Lcd_Line[19]={0x76,0x40};      	    	// Trame de texte compose de: {SLA+W=0x76, cde=write to DDRAM ,
						 	     								// le texte  afficher = 16 caractres max,  Nul }
                                                            	// Cette trame est remplie avec le tableau String_to_LCD
// =============================================================================================================================
// 3) Liste de messages prdfinis pour la carte MD23
// =============================================================================================================================
// 30) Initialisation
unsigned char Reset_MD23[]={MD23_ADD_BASE_W,Reg_Commande, Reset_Encodeur};

// 31) Mode
unsigned char Mode0_MD23[]={MD23_ADD_BASE_W,Reg_Mode,Mode0};

// 32) Vitesse des moteurs
unsigned char Speed1[]={MD23_ADD_BASE_W,Reg_Speed1, 128};		// Arrt moteur 1 dans le mode 0
unsigned char Speed2[]={MD23_ADD_BASE_W,Reg_Speed2_Turn, 128};  // Arrt moteur 2 dans le mode 0
unsigned char Time_OUT_Desable[]={MD23_ADD_BASE_W,Reg_Commande, Desactive_Timeout_Moteurs};

// 33) Lecture des registres
unsigned char Read_Reg_MD23_W[]= {MD23_ADD_BASE_W,0};		    // Premire trame  envoyer:
															    // { SLA_W=0xB0, N registre o commence la lecture }
// =============================================================================================================================
// 4) Trame de commande des servos
// ----------------------------------------------------------------------------------------------------------------------------- 
// Cette trame est compose de: {@SD21, NumServo, RegSpeedValue, RegPosLByte, RegPosHByte}
unsigned char Frame_I2C_Servo[]={SD21_ADD_BASE_W,Servo1,0,0xE8,0x03};		 

// Dtail du contenu de la "trame I2C Servo" 
// @SD21			  SLA+W = 0xC2: Adresse de la carte SD21 en criture
// NumServo			  N du registre vitesse du servo  commander, dfinit dans SD21.h (Servo1 par dfaut)
// RegSpeedValue	  Valeur  placer dans le le registre vitesse (Vitesse maxi par dfaut)
// RegPosLByte		  Octet de poids faible du mot de commande de la position (Position mini par dfaut)
// RegPosHByte		  Octet de poids fort du mot de commande de la position 
// =============================================================================================================================
// Variables diverses
// -----------------------------------------------------------------------------------------------------------------------------
// Type 			Nom 				       			   				Description
// -----------------------------------------------------------------------------------------------------------------------------
// 1) Gestion de la carte MD23
// -----------------------------------------------------------------------------------------------------------------------------
// Variables utilises dans la gestion de l'affichage:  limitent son rafrachissement
int Num_Reg_MD23_1=0, Num_Reg_MD23=0;                        		        // Numro du registre slectionn pour l'affichage
unsigned char Vitesse_MD23 = 0,Vitesse_MD23_1 = 1, BP_I2C=0, BP_I2C_1 = 1,  Finesse=5, Finesse_1=1;
unsigned char SpeedMot1=128, SpeedMot1_1=128, SpeedMot2=128, SpeedMot2_1=128; // Limite le rafrachissement de l'affichage
unsigned char Choix_Mode_1=1, Choix_Mode=0;
unsigned char Ubatt_1=0, Ubatt=0;									        // Tension alimentation carte MD23
unsigned char Version_1=0, Version=0;
// Reoit le contenu des registres de la carte MD23. Liste ci-dessous
// Speed1, Speed2_Turn, Enc1a, Enc1b, Enc1c, Enc1d, Enc2a, Enc2b, Enc2c, Enc2d, Ubatt, IMot1, IMot2, SoftRev, AccRate, Mode, Cde
unsigned char Table_Registers_MD23[]={MD23_ADD_BASE_R,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
// -----------------------------------------------------------------------------------------------------------------------------
// 2) Gestion du LCD BATRON
// -----------------------------------------------------------------------------------------------------------------------------
char Buffer_for_LCD_Line0[17], Buffer_for_LCD_Line1[17];       // Reoivent les deux lignes de texte  envoyer  l'afficheur
															    // (16 caractres max. + Nul)
// -----------------------------------------------------------------------------------------------------------------------------
// 3) Gestion de la carte SD21
// -----------------------------------------------------------------------------------------------------------------------------
unsigned char Vitesse = 5;				    // Vitesse de dplacement du servo          
unsigned int tempo = 1000;                  // Attente entre deux transmission : doit tre > 20ms 
unsigned int i = 0;						    // Variable de boucle 
// -----------------------------------------------------------------------------------------------------------------------------
// *****************************************************************************************************************************

// =============================================================================================================================
// ===================================================== Initialisations =======================================================
// =============================================================================================================================
TWI_Master_Initialise(); 									    // Initialisation de l'interface TWI en mode Standby
// -----------------------------------------------------------------------------------------------------------------------------
// Global enable interrupts
#asm("sei")							// L'autorisation des interruptions doit se faire aprs l'initialisation de l'interface TWI
// -----------------------------------------------------------------------------------------------------------------------------
TWI_Start_Transc_With_Data(Frame_I2C_LED_All_Off,sizeof(Frame_I2C_LED_All_Off)); // Extinction des LEDs de la carte S_ISI
// -----------------------------------------------------------------------------------------------------------------------------
// Initialisation de l'afficheur I2C BATRON
// -----------------------------------------------------------------------------------------------------------------------------
Init_LCD_I2C_BATRON();
// -----------------------------------------------------------------------------------------------------------------------------
// Affichage d'un message d'accueil
// -----------------------------------------------------------------------------------------------------------------------------
sprintf(Buffer_for_LCD_Line0,Texte_Accueil_Ligne1);
sprintf(Buffer_for_LCD_Line1,Texte_Accueil_Ligne2);
Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
delay_ms(4000);
// -----------------------------------------------------------------------------------------------------------------------------
// Initialisation des registres de  la carte MD23
// -----------------------------------------------------------------------------------------------------------------------------
TWI_Start_Transc_With_Data(Reset_MD23,sizeof(Reset_MD23));
TWI_Start_Transc_With_Data(Time_OUT_Desable,sizeof(Time_OUT_Desable));
TWI_Start_Transc_With_Data(Mode0_MD23,sizeof(Mode0_MD23));
TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
// =============================================================================================================================

// =============================================================================================================================
// ================================================ Boucle principale ==========================================================
// =============================================================================================================================
while (1)
      {
// ---------------------------------------------  Champ Lecture des donnes ----------------------------------------------------
// Lecture de l'tat des boutons-poussoir (BP)
// ----------------------------------------------------------------------------------------------------------------------------
// Lecture des BP pour affichage sur les LED
	TWI_Start_Transc_With_Data(Frame_I2C_BP_Read,1);            // Envoi de l'octet d'adresse des BP: SLA+R=0x43
	                                                            // Rcupration de l'octet reprsentatif de l'tat des BP
	TWI_Get_Data_From_Transceiver(Frame_I2C_BP_Read,sizeof(Frame_I2C_BP_Read));
// Lecture des BP avec gestion anti-rebond pour parcourir les menus
    BP_I2C = Lire_BP_I2C();
// ----------------------------------------------------------------------------------------------------------------------------
// Lecture des registres de la carte MD23
   if (Menu != Reglage_Valeurs_Mode0_MD23)
   {                                                           // Envoi de l'octet d'adresse MD23 + bit RW=criture + n du Reg
	TWI_Start_Transc_With_Data(Read_Reg_MD23_W,sizeof(Read_Reg_MD23_W));
   	TWI_Start_Transc_With_Data(Table_Registers_MD23,18);  		// Envoi octet d'adresse MD23 +  bit RW=lecture
	TWI_Get_Data_From_Transceiver(Table_Registers_MD23,sizeof(Table_Registers_MD23));
   }
// ----------------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------- Champ Traitement--------------------------------------------------------
    switch(Menu)
    {// Dbut du switch1
// -------------------------------------------------Dbut Menus droulants-----------------------------------------------------
// Menus Niveau 1 -------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------------------
// Menu 1 : Choix du test1: "Recopie de l'tat des BP sur les LED"
       case Menu1_LED_BP : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteMenuLedBp);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEntr);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
       		}
       		switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Copie_BP_Sur_Led;  break;     // touche Enter
					case FlecheH_I2C: Menu = Menu3_SD21;  break;        // touche flche haut
					case FlecheB_I2C: Menu = Menu2_MD23;  break;        // touche flche bas
					default: Menu = Menu1_LED_BP;
				}

			Menu_1 = Menu1_LED_BP;

       break;
// ----------------------------------------------------------------------------------------------------------------------------
// Menu 2 : Choix du test2: "Test de la carte moteur MD23/25"
       case Menu2_MD23 : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteMenuMD23);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEntr);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
       		}
       		switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Menu21_Mode_MD23;  break;     // touche Enter
					case FlecheH_I2C: Menu = Menu1_LED_BP;  break;      // touche flche haut
					case FlecheB_I2C: Menu = Menu3_SD21;  break;        // touche flche bas
					default: Menu = Menu2_MD23 ;
				}

			Menu_1 = Menu2_MD23 ;

       break;
// ----------------------------------------------------------------------------------------------------------------------------
// Menu 3 : Choix du test3: "Test de la carte SD21"
       case Menu3_SD21 : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteMenuSD21);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEntr);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
       		}
       		switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Test_SD21;  break;           // touche Enter
					case FlecheH_I2C: Menu = Menu2_MD23;  break;        // touche flche haut
					case FlecheB_I2C: Menu = Menu1_LED_BP;  break;      // touche flche bas
					default: Menu = Menu3_SD21 ;
				}

			Menu_1 = Menu3_SD21 ;

       break;
// ----------------------------------------------------------------------------------------------------------------------------
// Sous menus(Niveau 2) -------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------------------
// Sous menu 21: Choix du mode de commande des moteurs de la carte MD23/25
		case Menu21_Mode_MD23 : if (Menu_1 != Menu)
			{
				sprintf(Buffer_for_LCD_Line0,Ligne0MenuMode);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchapOk);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				LCD_I2C_BATRON_gotoxy(0,15);
				Built_Strng_Fram_I2C_LCDBATRON(Frame_I2C_Lcd_Line, Buffer_for_LCD_Line0, strlen(Buffer_for_LCD_Line0));
				TWI_Start_Transc_With_Data(Frame_I2C_Lcd_Line,strlen(Frame_I2C_Lcd_Line));
				Display_Arrow_0n_I2C_LCDBATRON();
			}

			switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Choix_Mode_MD23;  break;         // touche Enter
					case FlecheH_I2C: Menu = Menu22_Infos_MD23;  break;    // touche flche haut
					case FlecheB_I2C: Menu = Menu22_Infos_MD23;  break;    // touche flche bas
					case ECHAP_I2C : Menu = Menu2_MD23; break;             // touche ECHAP
					default: Menu = Menu21_Mode_MD23 ;
				}

			Menu_1 = Menu21_Mode_MD23;
		break;
// ----------------------------------------------------------------------------------------------------------------------------
// Sous menu 22 : Choix de l'affichage du contenu des registres de la carte MD23/25
		case Menu22_Infos_MD23: if (Menu_1 != Menu)
			{
				sprintf(Buffer_for_LCD_Line0,Ligne0MenuInfos);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEntr);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
			}

			switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Info_All;  break;
					case FlecheH_I2C: Menu = Menu21_Mode_MD23;  break;
					case FlecheB_I2C: Menu = Menu21_Mode_MD23;  break;
					case ECHAP_I2C : Menu = Menu2_MD23; break;              // touche ECHAP
					default: Menu = Menu22_Infos_MD23 ;
				}

			Menu_1 = Menu22_Infos_MD23 ;
		break;
// ----------------------------------------------------------------------------------------------------------------------------
// Tests ----------------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------------------
// Test1: Affichage du nom du BP et de la LED  laquelle il est associ sur le LCD
			case Copie_BP_Sur_Led : if ((Menu_1 != Menu) || ((BP_I2C != BP_I2C_1) && (BP_I2C != 0xFF)))
				{
				 sprintf(Buffer_for_LCD_Line0,TexteTest1BpsurLed);

				 switch (BP_I2C)
				  {
				 	case ENTR_I2C : sprintf(Buffer_for_LCD_Line1,BpEntrSurD6); break;
				 	case FlecheH_I2C : sprintf(Buffer_for_LCD_Line1,BpFlecheHSurD1); break;
				 	case FlecheB_I2C : sprintf(Buffer_for_LCD_Line1,BpFlecheHSurD2); break;
				 	case ECHAP_I2C : sprintf(Buffer_for_LCD_Line1,BpEchSurD7); break;
				 	case OK_I2C : sprintf(Buffer_for_LCD_Line1,BpOkSurD5); break;
				 	case MOINS_I2C: sprintf(Buffer_for_LCD_Line1,BpMoinsSurD3); break;
				 	case PLUS_I2C: sprintf(Buffer_for_LCD_Line1,BpPlusSurD0); break;
				 	case SET_I2C: sprintf(Buffer_for_LCD_Line1,BpSetSurD4); break;
				 	case 0 : sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchap );
				  }

					Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
					BP_I2C_1 = BP_I2C;
			   	 }

        		if (BP_I2C == ECHAP_I2C)
        		  {
        			Frame_I2C_LED_Switching[1] = ~Frame_I2C_BP_Read[1];
        			TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,2);  	// Les LED affichent l'tat des BP
        			delay_ms(1000);
        			Menu = Menu1_LED_BP;
        		  }
        		else  Menu = Copie_BP_Sur_Led;

			    Menu_1 = Copie_BP_Sur_Led;
			break;
// ----------------------------------------------------------------------------------------------------------------------------
// Test 2.1: Rglage du mode de commande des moteurs
	        case Choix_Mode_MD23: if ((Menu_1 != Menu) || (Choix_Mode_1 != Choix_Mode))
	          {
	        	 sprintf(Buffer_for_LCD_Line0,"Choix: Mode%-u    ",Choix_Mode);
	        	 sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchapOk);
	        	 Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
	        	 Display_Arrow_0n_I2C_LCDBATRON();
	        	 Choix_Mode_1 = Choix_Mode;
	          }

	          switch (BP_I2C)
				{
					case ECHAP_I2C : Menu=Menu21_Mode_MD23; break;  // touche ECHAP
					case FlecheH_I2C: if (Choix_Mode < 3) Choix_Mode = Choix_Mode + 1; else Choix_Mode=0;  break;
					case FlecheB_I2C: if (Choix_Mode > 0) Choix_Mode = Choix_Mode - 1; else Choix_Mode=3;  break;
					case ENTR_I2C: if (Choix_Mode == 0) {SpeedMot1 = 128; SpeedMot2 = 128; Menu = Reglage_Valeurs_Mode0_MD23;};	break;

					default: Menu=Choix_Mode_MD23;
				}

			  Menu_1 = Choix_Mode_MD23;
	        break;
// ----------------------------------------------------------------------------------------------------------------------------
            case Reglage_Valeurs_Mode0_MD23:   if ((Menu_1 != Menu) || (SpeedMot1_1 != SpeedMot1)||(SpeedMot2_1 != SpeedMot2)||(Finesse_1 != Finesse))
              {
                sprintf(Buffer_for_LCD_Line0,"1.%-u Mode%-u (+%-u)  ",Choix_Mode+1,Choix_Mode,Finesse);
                sprintf(Buffer_for_LCD_Line1,"SM1=%-u SM2=%-u",SpeedMot1,SpeedMot2);
                Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
                Display_Arrow_0n_I2C_LCDBATRON();
                SpeedMot1_1 = SpeedMot1; SpeedMot2_1 = SpeedMot2; Finesse_1 = Finesse;
              }


              switch (BP_I2C)
				{
				 	case ECHAP_I2C :  Menu=Choix_Mode_MD23; break;

				 	case PLUS_I2C:
				 		switch (Choix_Moteur)
				 		  {
				 		   	case Moteur1: if (Finesse==1)
				 		   					if (SpeedMot1 < 255) SpeedMot1++; else SpeedMot1=255;
				 		   			else
				 		   					if (SpeedMot1 < 250) SpeedMot1+=5; else SpeedMot1=255;
				 		   	break;
				 		   	case Moteur2: if (Finesse==1)
				 		   					if (SpeedMot2 < 255) SpeedMot2++; else SpeedMot2=255;
				 		   			else
				 		   					if (SpeedMot2 < 250) SpeedMot2+=5; else SpeedMot2=255;
				 		   	break;
				 		  }
				 	break;

				 	case MOINS_I2C:
				 	    switch (Choix_Moteur)
				 		  {
				 		   	case Moteur1: if (Finesse==1)
				 		   					if (SpeedMot1 > 0) SpeedMot1--; else SpeedMot1=0;
				 		   			else
				 		   					if (SpeedMot1 > 5) SpeedMot1-=5; else SpeedMot1=0;
				 		   	break;
				 		   	case Moteur2: if (Finesse==1)
				 		   					if (SpeedMot2 > 0) SpeedMot2--; else SpeedMot2=0;
				 		   			else
				 		   					if (SpeedMot2 > 5) SpeedMot2-=5; else SpeedMot2=0;
				 			break;
				 		  }
				 	break;

				 	case FlecheH_I2C: if (Finesse==1) Finesse = 5; else Finesse = 1; break;

				 	case FlecheB_I2C: if (Finesse==1) Finesse = 5; else Finesse = 1; break;

				 	case OK_I2C: if (Choix_Moteur == 1) Choix_Moteur=0; else Choix_Moteur=1; break;

				 	case SET_I2C:   Speed1[2] = SpeedMot1; Speed2[2] = SpeedMot2;
				 				    TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
									TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
				    break;

					default: Menu = Reglage_Valeurs_Mode0_MD23;
				}
			  Menu_1 = Reglage_Valeurs_Mode0_MD23;
			break;
// ----------------------------------------------------------------------------------------------------------------------------
// Test 2.2.1: Affichage du contenu des registres de la carte MD23/MD25
		case Info_All: Vitesse_MD23 = Table_Registers_MD23[6];
			if ((Menu_1 != Menu)||(Num_Reg_MD23_1 != Num_Reg_MD23)||(Vitesse_MD23_1 != Vitesse_MD23))
			  {
				sprintf(Buffer_for_LCD_Line0,"2.2.1 Reg%-u= %-u",Num_Reg_MD23,Table_Registers_MD23[Num_Reg_MD23+1]) ;
				strcpyf(Buffer_for_LCD_Line1,Ligne1PlusMoins);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
				Num_Reg_MD23_1 = Num_Reg_MD23; Vitesse_MD23_1 = Vitesse_MD23;
			  }

			switch (BP_I2C)
				{
					case ECHAP_I2C : Menu=Menu22_Infos_MD23; break;
					case PLUS_I2C: if (Num_Reg_MD23<16) Num_Reg_MD23++; else Num_Reg_MD23=0;  break;
					case MOINS_I2C: if (Num_Reg_MD23>0) Num_Reg_MD23--; else Num_Reg_MD23=16;  break;
					case FlecheH_I2C: Menu = Info_TensionBatt;  break;
					case FlecheB_I2C: Menu = Info_Version;  break;
					default: Menu=Info_All;
				}

			Menu_1 = Info_All;
		break;
// ----------------------------------------------------------------------------------------------------------------------------
// Test 2.2.2: Affichage de la version du logiciel implant dans la carte MD23, MD25
		case Info_Version:	Version = Table_Registers_MD23[14];
			if ((Menu_1 != Menu)||(Version_1 != Version))
   			  {
				sprintf(Buffer_for_LCD_Line0,"2.2.2 Version=%-u",Version);
				strcpyf(Buffer_for_LCD_Line1,Ligne1ChoixEchap);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
				Version_1 = Version;
			  }

			switch (BP_I2C)
				{
					case ECHAP_I2C : Menu=Menu22_Infos_MD23; break;
					case FlecheH_I2C: Menu = Info_All;  break;
					case FlecheB_I2C: Menu = Info_TensionBatt;  break;
					default: Menu=Info_Version;
				}

			Menu_1 = Info_Version;
		break;
// ----------------------------------------------------------------------------------------------------------------------------
// Test 2.2.3: Affichage de la tension d'alimentation de la batterie
		case Info_TensionBatt:  	Ubatt = Table_Registers_MD23[11];
			if ((Menu_1 != Menu)||(Ubatt_1 != Ubatt))
   			  {
					sprintf(Buffer_for_LCD_Line0,"2.2.3 Ubatt=%-u.%-uV",Ubatt/10, Ubatt-10*(Ubatt/10));
					strcpyf(Buffer_for_LCD_Line1,Ligne1ChoixEchap);
					Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
			  }

			switch (BP_I2C)
				{
					case ECHAP_I2C : Menu=Menu22_Infos_MD23; break;
					case FlecheH_I2C: Menu = Info_Version;  break;
					case FlecheB_I2C: Menu = Info_All;  break;
					default: Menu=Info_TensionBatt;
				}

			Menu_1 = Info_TensionBatt;
		break;
// ----------------------------------------------------------------------------------------------------------------------------
// Test3: Commande de quatre servos relis  la carte SD21
        case Test_SD21 : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteTestSD21);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchap5s);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
       		}  
            
            // Position initial du servo1
            RegSpeedValue = Vitesse; RegPosLByte= 0xE8; RegPosHByte=0x03;  					// 1000s = 03E8h
            TWI_Start_Transc_With_Data(Frame_I2C_Servo,sizeof(Frame_I2C_Servo));			// Vitesse, LSByte, MSByte                                                                                                                                           
            // =================================================================================================================
            // ================================================ Boucle  ========================================================
            // ================================================================================================================= 
            // Pour info
            // 1000 = 3E8h	valeur min
            // 1250 = 4E2h
            // 1500 = 5DCh
            // 1750 = 6D6h 
            // 1950 = 7E9h et + ?
            // -----------------------------------------------------------------------------------------------------------------
            // Les valeurs n Servo + Position transmises  la carte SD21 par le bus I2C  doivent tre places dans 
            // les tableaux situs dans le fichier "TableRob.h"
            // Le nom des servos est dclar dans le fichier sd21.h  
            
            while (BP_I2C != ECHAP_I2C) // Test des servos tant que l'on appui pas sur ECHAP
            { 
                // Configuration des paramtres "Servox" [0<x<5] et "position" envoys aux servos 
                for(i=0; i<sizeof(Tab_Servo); i++)
                {
                // Configuration de la trame "Frame_I2C_Servo" avec les valeurs contenues dans les tableaux situs dans 
                // le fichier Table_Rob.c
                NumServo = Tab_Servo[i]; RegPosLByte = LSBYTE(Tab_Position[i]);  RegPosHByte = MSBYTE(Tab_Position[i]); 
     	
	            // Envoi de la trame  "Frame_I2C_Servo"  la carte SD21
	            TWI_Start_Transc_With_Data(Frame_I2C_Servo,sizeof(Frame_I2C_Servo));	      
	
	            // Visualisation de la position des valeurs dans les tableaux sur les Leds de la carte Sisi
	            Frame_I2C_LED_Switching[1] = ~(i+1);
	            TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,sizeof(Frame_I2C_LED_Switching));
	            delay_ms(tempo);   
                } // fin du for
                 
                BP_I2C = Lire_BP_I2C(); //   
            } 
            // Servo maintenus dans leur dernire position
            RegSpeedValue = 0; TWI_Start_Transc_With_Data(Frame_I2C_Servo,sizeof(Frame_I2C_Servo));
                  
			Menu = Menu3_SD21 ; Menu_1 = Test_SD21 ;
            
       break;  
// ----------------------------------------------------------------------------------------------------------------------------
    }// Fin du switch1

    Frame_I2C_LED_Switching[1] = ~Frame_I2C_BP_Read[1];
// ----------------------------------------------------------------------------------------------------------------------------
// ----------------------------------------  Champ Ecriture des rsultats  ----------------------------------------------------
// Ecriture sur les LED
    TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,sizeof(Frame_I2C_LED_Switching));  	// Les LED affichent l'tat des BP
// ----------------------------------------------------------------------------------------------------------------------------
//delay_ms(100);   // A utiliser pour visualiser les trames I2C
  }; // Fin while(1)

} //Fin main


